from .packet import Packet

class PacketBuilder():
    ''' Builds a command packet using the builder design pattern '''

    def __init__(self, ip, port):
        self.ip = ip
        self.port = port
        self.cmd = None
        self.args = list()
        self.params = list()
        self.keys = dict()

    def addCommand(self, command):
        self.cmd = command

        return self

    def addArgument(self, argument):
        self.args.append(argument)

        return self
    
    def addArguments(self, *arguments):
        if arguments:
            for argument in arguments:
                self.args.append(argument)

        return self

    def addParameter(self, parameter):
        self.params.append(parameter)

        return self

    def addParameters(self, *parameters):
        if parameters:
            for parameter in parameters:
                self.params.append(parameter)

        return self

    def addKeyValuePair(self, key, value):
        self.keys[key] = value

        return self

    def _formatParameter(self, param):
        if not param.startswith('--'):
            if  param.startswith('-'):
                param = '-' + param
            else:
                param = '--' + param

        return param

    def build(self):
        if self.cmd == None: 
            return None

        query = "%s?" % self.cmd

        if self.args: 
            query = "%s%s&" % (query, '&'.join(self.args))

        if self.params: 
            params = list(map(self._formatParameter, self.params))
            query = "%s%s&" % (query, '&'.join(params))

        if self.keys:
            keys = list(map(lambda x: "x=%s" % self.keys[x], self.keys))
            query = "%s%s" % (query, '&'.join(keys))

        uri = '/'.join(['cmd', 'json'])
        uri = "%s/%s" % (uri, query.strip('&'))

        return Packet(self.ip, self.port, uri)


import requests
from .general_utils import print_error

class Packet:
    ''' represents a data packet '''

    def __init__(self, ip, port, query):
        self.ip = ip
        self.port = port
        self.query = query

    def send(self, timeout=5):
        response = None

        try:
            response = requests.get('http://%s:%s/%s' % (self.ip, self.port, self.query), timeout=timeout)
        except requests.exceptions.HTTPError as http_error:
            print_error('Warning: we have encountered in a http error.\n%s\n' % http_error)
            response = self._generateEmptyResponse('http')
        except requests.exceptions.Timeout as timeout_error:
            print_error('Warning: request timeout have been reached.\n%s\n' % timeout_error)
            response = self._generateEmptyResponse('timeout')
        except requests.exceptions.ConnectionError as connection_error:
            print_error('Warning: we have encountered in a connection error.\n%s\n' % connection_error)
            response = self._generateEmptyResponse('connection')
        except requests.exceptions.RequestException as error:
            print_error('Warning: request failed.\n%s\n' % error)
            response = self._generateEmptyResponse()

        return response

    @staticmethod
    def _generateEmptyResponse(errorType=b'unknown'):
        response = requests.Response()

        response._content = b'{ "exception": "Request resolved with an %s error" }' % errorType.encode("ascii")
        response.code = "expired"
        response.error_type = "expired"
        response.status_code = 400

        return response


from .utils.packet_builder import PacketBuilder

class Client():
    ''' Butter MAS HTTP client API '''

    def __init__(self, ip, port=5555):
        self.ip = ip
        self.port = port

    def getAvailableHandlers(self):
        packet = PacketBuilder(self.ip, self.port).addCommand('list').build()

        return packet.send()

    def getAvailableAnimations(self, reload=False):
        packet = PacketBuilder(self.ip, self.port).addCommand('animate').addParameter('list').build()

        return packet.send()

    def getAvailableSounds(self, reload=False):
        packet = PacketBuilder(self.ip, self.port).addCommand('audio').addParameter('list').build()

        return packet.send()

    def getAvailableMotorRegisters(self, motorName, writeableOnly=False):
        packet = PacketBuilder(self.ip, self.port).addCommand('get').addParameter('list').build()

        return packet.send()

    def getMotorRegister(self, motorName, registerName):
        packet = PacketBuilder(self.ip, self.port).addCommand('get').addArguments(motorName, registerName).build()

        return packet.send()

    def setMotorRegister(self, motorName, registerName, value):
        packet = PacketBuilder(self.ip, self.port).addCommand('set').addArguments(motorName, registerName, value).build()        

        return packet.send()

    def playAnimation(self, animationName):
        packet = PacketBuilder(self.ip, self.port).addCommand('animate').addArgument(animationName).build()        

        return packet.send()

    def pauseAnimation(self):
        packet = PacketBuilder(self.ip, self.port).addCommand('animate').addParameter('pause').build()        

        return packet.send()

    def resumeAnimation(self):
        packet = PacketBuilder(self.ip, self.port).addCommand('animate').addParameter('resume').build()        

        return packet.send()

    def stopAnimation(self):
        packet = PacketBuilder(self.ip, self.port).addCommand('animate').addParameter('stop').build()        

        return packet.send() 

    def playAudio(self, fileName):
        packet = PacketBuilder(self.ip, self.port).addCommand('audio').addArgument(fileName).build()        

        return packet.send()

    def pauseAudio(self):
        packet = PacketBuilder(self.ip, self.port).addCommand('audio').addParameter('pause').build()        

        return packet.send()

    def resumeAudio(self):
        packet = PacketBuilder(self.ip, self.port).addCommand('audio').addParameter('resume').build()        

        return packet.send()

    def stopAudio(self):
        packet = PacketBuilder(self.ip, self.port).addCommand('audio').addParameter('stop').build()        

        return packet.send()

if __name__ == "__main__":
    client = Client('localhost')

    print(client.getAvailableHandlers().json())


import unittest
from butter.mas.utils.packet import Packet

class TestPacketMethods(unittest.TestCase):

    def testSend(self):
        packet = Packet('localhost', 5555, 'cmd/json/help')
        self.assertIsNotNone(packet.send())

if __name__ == '__main__':
    unittest.main()